@@ -8,6 +8,7 @@ from moto.moto_api._internal import mock_random
 from moto.s3.models import s3_backends
 from moto.s3.utils import bucket_and_name_from_url
 from moto.utilities.paginator import paginate
+from moto.utilities.tagging_service import TaggingService
 from moto.utilities.utils import get_partition
 
 
@@ -117,6 +118,7 @@ class Execution(BaseModel):
         self.workgroup = workgroup
         self.execution_parameters = execution_parameters
         self.start_time = time.time()
+        self.end_time = time.time()
         self.status = "SUCCEEDED"
 
         if self.config is not None and "OutputLocation" in self.config:
@@ -139,6 +141,28 @@ class QueryResults(BaseModel):
         }
 
 
+class CapacityReservation(TaggableResourceMixin, BaseModel):
+    def __init__(
+        self,
+        athena_backend: "AthenaBackend",
+        name: str,
+        target_dpus: int,
+        tags: List[Dict[str, str]],
+    ):
+        self.region_name = athena_backend.region_name
+        super().__init__(
+            athena_backend.account_id,
+            self.region_name,
+            f"capacityreservation/{name}",
+            tags,
+        )
+        self.athena_backend = athena_backend
+        self.name = name
+        self.target_dpus = target_dpus
+        self.create_tags(tags)
+        self.tags = tags
+
+
 class NamedQuery(BaseModel):
     def __init__(
         self,
@@ -186,10 +210,12 @@ class AthenaBackend(BaseBackend):
         self.work_groups: Dict[str, WorkGroup] = {}
         self.executions: Dict[str, Execution] = {}
         self.named_queries: Dict[str, NamedQuery] = {}
+        self.capacity_reservations: Dict[str, CapacityReservation] = {}
         self.data_catalogs: Dict[str, DataCatalog] = {}
         self.query_results: Dict[str, QueryResults] = {}
         self.query_results_queue: List[QueryResults] = []
         self.prepared_statements: Dict[str, PreparedStatement] = {}
+        self.tagger = TaggingService()
 
         # Initialise with the primary workgroup
         self.create_work_group(
@@ -213,6 +239,7 @@ class AthenaBackend(BaseBackend):
             return None
         work_group = WorkGroup(self, name, configuration, description, tags)
         self.work_groups[name] = work_group
+        self.tagger.tag_resource(work_group.name, tags)
         return work_group
 
     def list_work_groups(self) -> List[Dict[str, Any]]:
@@ -366,6 +393,20 @@ class AthenaBackend(BaseBackend):
         execution = self.executions[exec_id]
         execution.status = "CANCELLED"
 
+    def create_capacity_reservation(
+        self,
+        name: str,
+        target_dpus: int,
+        tags: List[Dict[str, str]],
+    ) -> None:
+        cr = CapacityReservation(self, name, target_dpus, tags)
+        self.capacity_reservations[cr.name] = cr
+        self.tagger.tag_resource(cr.name, tags)
+        return None
+
+    def get_capacity_reservation(self, name: str) -> Optional[CapacityReservation]:
+        return self.capacity_reservations.get(name)
+
     def create_named_query(
         self,
         name: str,
@@ -418,6 +459,7 @@ class AthenaBackend(BaseBackend):
             self, name, catalog_type, description, parameters, tags
         )
         self.data_catalogs[name] = data_catalog
+        self.tagger.tag_resource(data_catalog.name, tags)
         return data_catalog
 
     @paginate(pagination_model=PAGINATION_MODEL)
@@ -459,5 +501,10 @@ class AthenaBackend(BaseBackend):
             return self.executions[query_execution_id]
         return None
 
+    def list_tags_for_resource(self, resource_arn: str) -> Optional[Dict[str, Any]]:
+        if self.tagger.has_tags(resource_arn):
+            return self.tagger.list_tags_for_resource(resource_arn)
+        return None
+
 
 athena_backends = BackendDict(AthenaBackend, "athena")
