@@ -38,6 +38,7 @@ if TYPE_CHECKING:
     from moto.cognitoidp.models import CognitoIdpBackend
     from moto.comprehend.models import ComprehendBackend
     from moto.config.models import ConfigBackend
+    from moto.connectcampaigns.models import ConnectCampaignServiceBackend
     from moto.core.base_backend import SERVICE_BACKEND, BackendDict
     from moto.databrew.models import DataBrewBackend
     from moto.datapipeline.models import DataPipelineBackend
@@ -106,7 +107,6 @@ if TYPE_CHECKING:
     from moto.networkmanager.models import NetworkManagerBackend
     from moto.opensearch.models import OpenSearchServiceBackend
     from moto.opensearchserverless.models import OpenSearchServiceServerlessBackend
-    from moto.opsworks.models import OpsWorksBackend
     from moto.organizations.models import OrganizationsBackend
     from moto.osis.models import OpenSearchIngestionBackend
     from moto.personalize.models import PersonalizeBackend
@@ -228,6 +228,7 @@ SERVICE_NAMES = Union[
     "Literal['cognito-identity']",
     "Literal['cognito-idp']",
     "Literal['comprehend']",
+    "Literal['connectcampaigns']",
     "Literal['config']",
     "Literal['databrew']",
     "Literal['datapipeline']",
@@ -296,7 +297,6 @@ SERVICE_NAMES = Union[
     "Literal['networkmanager']",
     "Literal['opensearch']",
     "Literal['opensearchserverless']",
-    "Literal['opsworks']",
     "Literal['organizations']",
     "Literal['osis']",
     "Literal['personalize']",
@@ -442,6 +442,10 @@ def get_backend(name: "Literal['cognito-idp']") -> "BackendDict[CognitoIdpBacken
 @overload
 def get_backend(name: "Literal['comprehend']") -> "BackendDict[ComprehendBackend]": ...
 @overload
+def get_backend(
+    name: "Literal['connectcampaigns']",
+) -> "BackendDict[ConnectCampaignServiceBackend]": ...
+@overload
 def get_backend(name: "Literal['config']") -> "BackendDict[ConfigBackend]": ...
 @overload
 def get_backend(name: "Literal['databrew']") -> "BackendDict[DataBrewBackend]": ...
@@ -634,8 +638,6 @@ def get_backend(
     name: "Literal['opensearchserverless']",
 ) -> "BackendDict[OpenSearchServiceServerlessBackend]": ...
 @overload
-def get_backend(name: "Literal['opsworks']") -> "BackendDict[OpsWorksBackend]": ...
-@overload
 def get_backend(
     name: "Literal['osis']",
 ) -> "BackendDict[OpenSearchIngestionBackend]": ...
