@@ -1,58 +1,69 @@
+from moto.core.responses import ActionResult, EmptyResult
+
 from ._base_response import EC2BaseResponse
 
 
 class ElasticBlockStore(EC2BaseResponse):
-    def attach_volume(self):
+    def attach_volume(self) -> str:
         volume_id = self._get_param("VolumeId")
         instance_id = self._get_param("InstanceId")
         device_path = self._get_param("Device")
-        if self.is_not_dryrun("AttachVolume"):
-            attachment = self.ec2_backend.attach_volume(
-                volume_id, instance_id, device_path
-            )
-            template = self.response_template(ATTACHED_VOLUME_RESPONSE)
-            return template.render(attachment=attachment)
 
-    def copy_snapshot(self):
+        self.error_on_dryrun()
+
+        attachment = self.ec2_backend.attach_volume(volume_id, instance_id, device_path)
+        template = self.response_template(ATTACHED_VOLUME_RESPONSE)
+        return template.render(attachment=attachment)
+
+    def copy_snapshot(self) -> str:
         source_snapshot_id = self._get_param("SourceSnapshotId")
         source_region = self._get_param("SourceRegion")
         description = self._get_param("Description")
         tags = self._parse_tag_specification()
         snapshot_tags = tags.get("snapshot", {})
-        if self.is_not_dryrun("CopySnapshot"):
-            snapshot = self.ec2_backend.copy_snapshot(
-                source_snapshot_id, source_region, description
-            )
-            snapshot.add_tags(snapshot_tags)
-            template = self.response_template(COPY_SNAPSHOT_RESPONSE)
-            return template.render(snapshot=snapshot)
+        kms_key_id = self._get_param("KmsKeyId")
 
-    def create_snapshot(self):
+        self.error_on_dryrun()
+
+        snapshot = self.ec2_backend.copy_snapshot(
+            source_snapshot_id=source_snapshot_id,
+            source_region=source_region,
+            description=description,
+            kms_key_id=kms_key_id,
+        )
+        snapshot.add_tags(snapshot_tags)
+        template = self.response_template(COPY_SNAPSHOT_RESPONSE)
+        return template.render(snapshot=snapshot)
+
+    def create_snapshot(self) -> str:
         volume_id = self._get_param("VolumeId")
         description = self._get_param("Description")
         tags = self._parse_tag_specification()
         snapshot_tags = tags.get("snapshot", {})
-        if self.is_not_dryrun("CreateSnapshot"):
-            snapshot = self.ec2_backend.create_snapshot(volume_id, description)
-            snapshot.add_tags(snapshot_tags)
-            template = self.response_template(CREATE_SNAPSHOT_RESPONSE)
-            return template.render(snapshot=snapshot)
 
-    def create_snapshots(self):
+        self.error_on_dryrun()
+
+        snapshot = self.ec2_backend.create_snapshot(volume_id, description)
+        snapshot.add_tags(snapshot_tags)
+        template = self.response_template(CREATE_SNAPSHOT_RESPONSE)
+        return template.render(snapshot=snapshot)
+
+    def create_snapshots(self) -> str:
         params = self._get_params()
         instance_spec = params.get("InstanceSpecification")
         description = params.get("Description", "")
         tags = self._parse_tag_specification()
         snapshot_tags = tags.get("snapshot", {})
 
-        if self.is_not_dryrun("CreateSnapshots"):
-            snapshots = self.ec2_backend.create_snapshots(
-                instance_spec, description, snapshot_tags
-            )
-            template = self.response_template(CREATE_SNAPSHOTS_RESPONSE)
-            return template.render(snapshots=snapshots)
+        self.error_on_dryrun()
 
-    def create_volume(self):
+        snapshots = self.ec2_backend.create_snapshots(
+            instance_spec, description, snapshot_tags
+        )
+        template = self.response_template(CREATE_SNAPSHOTS_RESPONSE)
+        return template.render(snapshots=snapshots)
+
+    def create_volume(self) -> str:
         size = self._get_param("Size")
         zone = self._get_param("AvailabilityZone")
         snapshot_id = self._get_param("SnapshotId")
@@ -61,32 +72,39 @@ class ElasticBlockStore(EC2BaseResponse):
         volume_tags = tags.get("volume", {})
         encrypted = self._get_bool_param("Encrypted", if_none=False)
         kms_key_id = self._get_param("KmsKeyId")
-        if self.is_not_dryrun("CreateVolume"):
-            volume = self.ec2_backend.create_volume(
-                size=size,
-                zone_name=zone,
-                snapshot_id=snapshot_id,
-                encrypted=encrypted,
-                kms_key_id=kms_key_id,
-                volume_type=volume_type,
-            )
-            volume.add_tags(volume_tags)
-            template = self.response_template(CREATE_VOLUME_RESPONSE)
-            return template.render(volume=volume)
+        iops = self._get_param("Iops")
+        throughput = self._get_param("Throughput")
 
-    def modify_volume(self):
+        self.error_on_dryrun()
+
+        volume = self.ec2_backend.create_volume(
+            size=size,
+            zone_name=zone,
+            snapshot_id=snapshot_id,
+            encrypted=encrypted,
+            kms_key_id=kms_key_id,
+            volume_type=volume_type,
+            iops=iops,
+            throughput=throughput,
+        )
+        volume.add_tags(volume_tags)
+        template = self.response_template(CREATE_VOLUME_RESPONSE)
+        return template.render(volume=volume)
+
+    def modify_volume(self) -> str:
         volume_id = self._get_param("VolumeId")
         target_size = self._get_param("Size")
         target_volume_type = self._get_param("VolumeType")
 
-        if self.is_not_dryrun("ModifyVolume"):
-            volume = self.ec2_backend.modify_volume(
-                volume_id, target_size, target_volume_type
-            )
-            template = self.response_template(MODIFY_VOLUME_RESPONSE)
-            return template.render(volume=volume)
+        self.error_on_dryrun()
 
-    def describe_volumes_modifications(self):
+        volume = self.ec2_backend.modify_volume(
+            volume_id, target_size, target_volume_type
+        )
+        template = self.response_template(MODIFY_VOLUME_RESPONSE)
+        return template.render(volume=volume)
+
+    def describe_volumes_modifications(self) -> str:
         filters = self._filters_from_querystring()
         volume_ids = self._get_multi_param("VolumeId")
         modifications = self.ec2_backend.describe_volumes_modifications(
@@ -95,19 +113,23 @@ class ElasticBlockStore(EC2BaseResponse):
         template = self.response_template(DESCRIBE_VOLUMES_MODIFICATIONS_RESPONSE)
         return template.render(modifications=modifications)
 
-    def delete_snapshot(self):
+    def delete_snapshot(self) -> ActionResult:
         snapshot_id = self._get_param("SnapshotId")
-        if self.is_not_dryrun("DeleteSnapshot"):
-            self.ec2_backend.delete_snapshot(snapshot_id)
-            return DELETE_SNAPSHOT_RESPONSE
 
-    def delete_volume(self):
+        self.error_on_dryrun()
+
+        self.ec2_backend.delete_snapshot(snapshot_id)
+        return EmptyResult()
+
+    def delete_volume(self) -> ActionResult:
         volume_id = self._get_param("VolumeId")
-        if self.is_not_dryrun("DeleteVolume"):
-            self.ec2_backend.delete_volume(volume_id)
-            return DELETE_VOLUME_RESPONSE
 
-    def describe_snapshots(self):
+        self.error_on_dryrun()
+
+        self.ec2_backend.delete_volume(volume_id)
+        return EmptyResult()
+
+    def describe_snapshots(self) -> str:
         filters = self._filters_from_querystring()
         snapshot_ids = self._get_multi_param("SnapshotId")
         snapshots = self.ec2_backend.describe_snapshots(
@@ -116,7 +138,7 @@ class ElasticBlockStore(EC2BaseResponse):
         template = self.response_template(DESCRIBE_SNAPSHOTS_RESPONSE)
         return template.render(snapshots=snapshots)
 
-    def describe_volumes(self):
+    def describe_volumes(self) -> str:
         filters = self._filters_from_querystring()
         volume_ids = self._get_multi_param("VolumeId")
         volumes = self.ec2_backend.describe_volumes(
@@ -125,73 +147,86 @@ class ElasticBlockStore(EC2BaseResponse):
         template = self.response_template(DESCRIBE_VOLUMES_RESPONSE)
         return template.render(volumes=volumes)
 
-    def describe_volume_attribute(self):
+    def describe_volume_attribute(self) -> str:
         raise NotImplementedError(
             "ElasticBlockStore.describe_volume_attribute is not yet implemented"
         )
 
-    def describe_volume_status(self):
+    def describe_volume_status(self) -> str:
         raise NotImplementedError(
             "ElasticBlockStore.describe_volume_status is not yet implemented"
         )
 
-    def detach_volume(self):
+    def detach_volume(self) -> str:
         volume_id = self._get_param("VolumeId")
         instance_id = self._get_param("InstanceId")
         device_path = self._get_param("Device")
-        if self.is_not_dryrun("DetachVolume"):
-            attachment = self.ec2_backend.detach_volume(
-                volume_id, instance_id, device_path
-            )
-            template = self.response_template(DETATCH_VOLUME_RESPONSE)
-            return template.render(attachment=attachment)
 
-    def enable_volume_io(self):
-        if self.is_not_dryrun("EnableVolumeIO"):
-            raise NotImplementedError(
-                "ElasticBlockStore.enable_volume_io is not yet implemented"
-            )
+        self.error_on_dryrun()
 
-    def import_volume(self):
-        if self.is_not_dryrun("ImportVolume"):
-            raise NotImplementedError(
-                "ElasticBlockStore.import_volume is not yet implemented"
-            )
+        attachment = self.ec2_backend.detach_volume(volume_id, instance_id, device_path)
+        template = self.response_template(DETATCH_VOLUME_RESPONSE)
+        return template.render(attachment=attachment)
 
-    def describe_snapshot_attribute(self):
+    def enable_volume_io(self) -> str:
+        self.error_on_dryrun()
+
+        raise NotImplementedError(
+            "ElasticBlockStore.enable_volume_io is not yet implemented"
+        )
+
+    def import_volume(self) -> str:
+        self.error_on_dryrun()
+
+        raise NotImplementedError(
+            "ElasticBlockStore.import_volume is not yet implemented"
+        )
+
+    def describe_snapshot_attribute(self) -> str:
         snapshot_id = self._get_param("SnapshotId")
         groups = self.ec2_backend.get_create_volume_permission_groups(snapshot_id)
         user_ids = self.ec2_backend.get_create_volume_permission_userids(snapshot_id)
         template = self.response_template(DESCRIBE_SNAPSHOT_ATTRIBUTES_RESPONSE)
         return template.render(snapshot_id=snapshot_id, groups=groups, userIds=user_ids)
 
-    def modify_snapshot_attribute(self):
+    def modify_snapshot_attribute(self) -> ActionResult:
         snapshot_id = self._get_param("SnapshotId")
         operation_type = self._get_param("OperationType")
         groups = self._get_multi_param("UserGroup")
         user_ids = self._get_multi_param("UserId")
-        if self.is_not_dryrun("ModifySnapshotAttribute"):
-            if operation_type == "add":
-                self.ec2_backend.add_create_volume_permission(
-                    snapshot_id, user_ids=user_ids, groups=groups
-                )
-            elif operation_type == "remove":
-                self.ec2_backend.remove_create_volume_permission(
-                    snapshot_id, user_ids=user_ids, groups=groups
-                )
-            return MODIFY_SNAPSHOT_ATTRIBUTE_RESPONSE
 
-    def modify_volume_attribute(self):
-        if self.is_not_dryrun("ModifyVolumeAttribute"):
-            raise NotImplementedError(
-                "ElasticBlockStore.modify_volume_attribute is not yet implemented"
-            )
+        self.error_on_dryrun()
 
-    def reset_snapshot_attribute(self):
-        if self.is_not_dryrun("ResetSnapshotAttribute"):
-            raise NotImplementedError(
-                "ElasticBlockStore.reset_snapshot_attribute is not yet implemented"
+        if operation_type == "add":
+            self.ec2_backend.add_create_volume_permission(
+                snapshot_id, user_ids=user_ids, groups=groups
             )
+        elif operation_type == "remove":
+            self.ec2_backend.remove_create_volume_permission(
+                snapshot_id, user_ids=user_ids, groups=groups
+            )
+        return EmptyResult()
+
+    def modify_volume_attribute(self) -> str:
+        self.error_on_dryrun()
+
+        raise NotImplementedError(
+            "ElasticBlockStore.modify_volume_attribute is not yet implemented"
+        )
+
+    def reset_snapshot_attribute(self) -> str:
+        self.error_on_dryrun()
+
+        raise NotImplementedError(
+            "ElasticBlockStore.reset_snapshot_attribute is not yet implemented"
+        )
+
+    def modify_ebs_default_kms_key_id(self) -> str:
+        self.error_on_dryrun()
+        kms_key_id = self._get_param("KmsKeyId")
+        new_default_kms_arn = self.ec2_backend.modify_ebs_default_kms_key_id(kms_key_id)
+        template = self.response_template(MODIFY_EBS_DEFAULT_KMS_KEY_ID_RESPONSE)
+        return template.render(kmsKeyId=new_default_kms_arn)
 
 
 CREATE_VOLUME_RESPONSE = """<CreateVolumeResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
@@ -209,7 +244,7 @@ CREATE_VOLUME_RESPONSE = """<CreateVolumeResponse xmlns="http://ec2.amazonaws.co
   {% endif %}
   <availabilityZone>{{ volume.zone.name }}</availabilityZone>
   <status>creating</status>
-  <createTime>{{ volume.create_time}}</createTime>
+  <createTime>{{ volume.create_time }}</createTime>
   {% if volume.get_tags() %}
     <tagSet>
       {% for tag in volume.get_tags() %}
@@ -223,6 +258,12 @@ CREATE_VOLUME_RESPONSE = """<CreateVolumeResponse xmlns="http://ec2.amazonaws.co
     </tagSet>
   {% endif %}
   <volumeType>{{ volume.volume_type }}</volumeType>
+  {% if volume.iops %}
+    <iops>{{ volume.iops }}</iops>
+  {% endif %}
+  {% if volume.throughput %}
+    <throughput>{{ volume.throughput }}</throughput>
+  {% endif %}
 </CreateVolumeResponse>"""
 
 DESCRIBE_VOLUMES_RESPONSE = """<DescribeVolumesResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
@@ -243,7 +284,7 @@ DESCRIBE_VOLUMES_RESPONSE = """<DescribeVolumesResponse xmlns="http://ec2.amazon
              {% endif %}
              <availabilityZone>{{ volume.zone.name }}</availabilityZone>
              <status>{{ volume.status }}</status>
-             <createTime>{{ volume.create_time}}</createTime>
+             <createTime>{{ volume.create_time }}</createTime>
              <attachmentSet>
                 {% if volume.attachment %}
                     <item>
@@ -269,15 +310,17 @@ DESCRIBE_VOLUMES_RESPONSE = """<DescribeVolumesResponse xmlns="http://ec2.amazon
                </tagSet>
              {% endif %}
              <volumeType>{{ volume.volume_type }}</volumeType>
+             {% if volume.iops %}
+               <iops>{{ volume.iops }}</iops>
+             {% endif %}
+             {% if volume.throughput %}
+               <throughput>{{ volume.throughput }}</throughput>
+             {% endif %}
           </item>
       {% endfor %}
    </volumeSet>
 </DescribeVolumesResponse>"""
 
-DELETE_VOLUME_RESPONSE = """<DeleteVolumeResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
-  <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId>
-  <return>true</return>
-</DeleteVolumeResponse>"""
 
 ATTACHED_VOLUME_RESPONSE = """<AttachVolumeResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
   <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId>
@@ -378,6 +421,7 @@ DESCRIBE_SNAPSHOTS_RESPONSE = """<DescribeSnapshotsResponse xmlns="http://ec2.am
             <volumeSize>{{ snapshot.volume.size }}</volumeSize>
              <description>{{ snapshot.description }}</description>
              <encrypted>{{ 'true' if snapshot.encrypted else 'false' }}</encrypted>
+             {% if snapshot.kms_key_id %}<kmsKeyId>{{ snapshot.kms_key_id }}</kmsKeyId>{% endif %}
              <tagSet>
                {% for tag in snapshot.get_tags() %}
                  <item>
@@ -393,10 +437,6 @@ DESCRIBE_SNAPSHOTS_RESPONSE = """<DescribeSnapshotsResponse xmlns="http://ec2.am
    </snapshotSet>
 </DescribeSnapshotsResponse>"""
 
-DELETE_SNAPSHOT_RESPONSE = """<DeleteSnapshotResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
-  <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId>
-  <return>true</return>
-</DeleteSnapshotResponse>"""
 
 DESCRIBE_SNAPSHOT_ATTRIBUTES_RESPONSE = """
 <DescribeSnapshotAttributeResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
@@ -417,12 +457,6 @@ DESCRIBE_SNAPSHOT_ATTRIBUTES_RESPONSE = """
 </DescribeSnapshotAttributeResponse>
 """
 
-MODIFY_SNAPSHOT_ATTRIBUTE_RESPONSE = """
-<ModifySnapshotAttributeResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
-    <requestId>666d2944-9276-4d6a-be12-1f4ada972fd8</requestId>
-    <return>true</return>
-</ModifySnapshotAttributeResponse>
-"""
 
 MODIFY_VOLUME_RESPONSE = """<?xml version="1.0" encoding="UTF-8"?>
 <ModifyVolumeResponse xmlns="http://ec2.amazonaws.com/doc/2016-11-15/">
@@ -460,3 +494,11 @@ DESCRIBE_VOLUMES_MODIFICATIONS_RESPONSE = """
       {% endfor %}
     </volumeModificationSet>
 </DescribeVolumesModificationsResponse>"""
+
+
+MODIFY_EBS_DEFAULT_KMS_KEY_ID_RESPONSE = """
+<ModifyEbsDefaultKmsKeyIdResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
+    <requestId>666d2944-9276-4d6a-be12-1f4ada972fd8</requestId>
+    <kmsKeyId>{{ kmsKeyId }}</kmsKeyId>
+</ModifyEbsDefaultKmsKeyIdResponse>
+"""
