@@ -34,10 +34,12 @@ class IoTResponse(BaseResponse):
         thing_name = self._get_param("thingName")
         thing_type_name = self._get_param("thingTypeName")
         attribute_payload = self._get_param("attributePayload")
+        billing_group_name = self._get_param("billingGroupName")
         thing = self.iot_backend.create_thing(
             thing_name=thing_name,
             thing_type_name=thing_type_name,
             attribute_payload=attribute_payload,
+            billing_group_name=billing_group_name,
         )
         return json.dumps(
             dict(thingName=thing.thing_name, thingArn=thing.arn, thingId=thing.thing_id)
@@ -59,9 +61,8 @@ class IoTResponse(BaseResponse):
             "maxResults", 50
         )  # not the default, but makes testing easier
         thing_type_name = self._get_param("thingTypeName")
-        thing_types = self.iot_backend.list_thing_types(thing_type_name=thing_type_name)
-
-        thing_types = [_.to_dict() for _ in thing_types]
+        _types = self.iot_backend.list_thing_types(thing_type_name=thing_type_name)
+        thing_types = [_.to_dict() for _ in _types]
         if previous_next_token is None:
             result = thing_types[0:max_results]
             next_token = str(max_results) if len(thing_types) > max_results else None
@@ -231,7 +232,9 @@ class IoTResponse(BaseResponse):
             max_results=max_results, next_token=previous_next_token
         )
 
-        return json.dumps(dict(jobs=jobs, nextToken=next_token))
+        return json.dumps(
+            dict(jobs=[job.to_dict() for job in jobs], nextToken=next_token)
+        )
 
     def describe_job_execution(self) -> str:
         job_id = self._get_param("jobId")
@@ -280,7 +283,12 @@ class IoTResponse(BaseResponse):
             job_id=job_id, status=status, max_results=max_results, token=next_token
         )
 
-        return json.dumps(dict(executionSummaries=job_executions, nextToken=next_token))
+        return json.dumps(
+            dict(
+                executionSummaries=[je.to_dict() for je in job_executions],
+                nextToken=next_token,
+            )
+        )
 
     def list_job_executions_for_thing(self) -> str:
         thing_name = self._get_param("thingName")
@@ -296,7 +304,12 @@ class IoTResponse(BaseResponse):
             next_token=next_token,
         )
 
-        return json.dumps(dict(executionSummaries=job_executions, nextToken=next_token))
+        return json.dumps(
+            dict(
+                executionSummaries=[je.to_dict() for je in job_executions],
+                nextToken=next_token,
+            )
+        )
 
     def create_keys_and_certificate(self) -> str:
         set_as_active = self._get_bool_param("setAsActive")
@@ -900,3 +913,82 @@ class IoTResponse(BaseResponse):
                 "jobExecutionsRetryConfig": job_template.job_execution_retry_config,
             }
         )
+
+    def create_billing_group(self) -> str:
+        billing_group_name = self._get_param("billingGroupName")
+        billing_group_properties = self._get_param("billingGroupProperties")
+        billing_group = self.iot_backend.create_billing_group(
+            billing_group_name=billing_group_name,
+            billing_group_properties=billing_group_properties,
+        )
+        return json.dumps(billing_group.to_dict())
+
+    def describe_billing_group(self) -> str:
+        billing_group_name = self._get_param("billingGroupName")
+        billing_group = self.iot_backend.describe_billing_group(
+            billing_group_name=billing_group_name
+        )
+        return json.dumps(billing_group.to_description_dict())
+
+    def delete_billing_group(self) -> str:
+        billing_group_name = self._get_param("billingGroupName")
+
+        self.iot_backend.delete_billing_group(billing_group_name=billing_group_name)
+        return json.dumps(dict())
+
+    def list_billing_groups(self) -> str:
+        name_prefix_filter = self._get_param("namePrefixFilter")
+        max_results = self._get_int_param("maxResults", 100)
+        token = self._get_param("nextToken")
+
+        billing_groups, next_token = self.iot_backend.list_billing_groups(
+            name_prefix_filter=name_prefix_filter, max_results=max_results, token=token
+        )
+
+        return json.dumps(dict(billingGroups=billing_groups, nextToken=next_token))
+
+    def update_billing_group(self) -> str:
+        billing_group_name = self._get_param("billingGroupName")
+        billing_group_properties = self._get_param("billingGroupProperties")
+        expected_version = self._get_param("expectedVersion")
+
+        version = self.iot_backend.update_billing_group(
+            billing_group_name=billing_group_name,
+            billing_group_properties=billing_group_properties,
+            expected_version=expected_version,
+        )
+
+        return json.dumps(dict(version=version))
+
+    def add_thing_to_billing_group(self) -> str:
+        self.iot_backend.add_thing_to_billing_group(
+            billing_group_name=self._get_param("billingGroupName"),
+            billing_group_arn=self._get_param("billingGroupArn"),
+            thing_name=self._get_param("thingName"),
+            thing_arn=self._get_param("thingArn"),
+        )
+
+        return json.dumps(dict())
+
+    def remove_thing_from_billing_group(self) -> str:
+        self.iot_backend.remove_thing_from_billing_group(
+            billing_group_name=self._get_param("billingGroupName"),
+            billing_group_arn=self._get_param("billingGroupArn"),
+            thing_name=self._get_param("thingName"),
+            thing_arn=self._get_param("thingArn"),
+        )
+
+        return json.dumps(dict())
+
+    def list_things_in_billing_group(self) -> str:
+        billing_group_name = self._get_param("billingGroupName")
+        max_results = self._get_int_param("maxResults", 100)
+        token = self._get_param("nextToken")
+        things, next_token = self.iot_backend.list_things_in_billing_group(
+            billing_group_name=billing_group_name,
+            max_results=max_results,
+            token=token,
+        )
+        return json.dumps(
+            dict(things=[thing.thing_name for thing in things], nextToken=next_token)
+        )
